'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _lodash = require('lodash.maxby');

var _lodash2 = _interopRequireDefault(_lodash);

var _initializeOptions = require('../initializeOptions/initializeOptions');

var _constants = require('../util/constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var setOption = function setOption(_, _ref) {
  var key = _ref.key,
      value = _ref.value;
  return _defineProperty({}, key, value);
};

var setPlaylist = function setPlaylist(_, _ref3) {
  var playlist = _ref3.playlist;
  return {
    current: 0,
    shuffled: false,
    playlist: playlist.map(function (media, index) {
      return _extends({}, media, {
        shufflePosition: index
      });
    })
  };
};

var add = function add(jPlaylist, _ref4) {
  var media = _ref4.media,
      playNow = _ref4.playNow;

  var highestMediaShufflePosition = (0, _lodash2.default)(jPlaylist.playlist, 'shufflePosition');
  var newMedia = _extends({}, media, {
    shufflePosition: highestMediaShufflePosition !== undefined ? highestMediaShufflePosition.shufflePosition + 1 : 0
  });
  var playlist = [].concat(_toConsumableArray(jPlaylist.playlist), [newMedia]);
  var current = void 0;

  if (playNow) {
    current = playlist.length - 1;
  } else if (playlist.length === 1) {
    current = 0;
  }

  return {
    playlist: playlist,
    playNow: playNow,
    current: current
  };
};

var remove = function remove(jPlaylist, _ref5) {
  var index = _ref5.index;

  var playlist = [].concat(_toConsumableArray(jPlaylist.playlist));
  var current = jPlaylist.current;
  var shuffled = jPlaylist.shuffled;

  playlist.splice(index, 1);

  if (playlist.length) {
    if (index === jPlaylist.current) {
      // To cope when last element being selected when it was removed
      current = index < playlist.length ? jPlaylist.current : playlist.length - 1;
    } else if (index < jPlaylist.current) {
      current -= 1;
    }
  } else {
    shuffled = false;
  }

  return {
    playlist: playlist,
    current: current,
    shuffled: shuffled
  };
};

var clear = function clear() {
  return {
    playlist: []
  };
};

// Negative index relates to the end of the array
var select = function select(jPlaylist, _ref6) {
  var index = _ref6.index;
  return {
    current: index < 0 ? jPlaylist.playlist.length + index : index
  };
};

// Negative index relates to the end of the array
var play = function play(jPlaylist, _ref7) {
  var _ref7$index = _ref7.index,
      index = _ref7$index === undefined ? jPlaylist.current : _ref7$index;
  return {
    current: index < 0 ? jPlaylist.playlist.length + index : index,
    playNow: true
  };
};

var shuffle = function shuffle(jPlaylist, _ref8) {
  var _ref8$shuffled = _ref8.shuffled,
      shuffled = _ref8$shuffled === undefined ? !jPlaylist.shuffled : _ref8$shuffled,
      playNow = _ref8.playNow,
      shuffleSort = _ref8.shuffleSort;

  var playlist = void 0;

  if (shuffled) {
    playlist = [].concat(_toConsumableArray(jPlaylist.playlist)).sort(function () {
      return shuffleSort;
    });
  } else {
    playlist = [].concat(_toConsumableArray(jPlaylist.playlist)).sort(function (a, b) {
      return a.shufflePosition - b.shufflePosition;
    });
  }

  return {
    playlist: playlist,
    shuffled: shuffled,
    playNow: playNow
  };
};

var next = function next(jPlaylist) {
  var current = jPlaylist.loop === 'loop-playlist' ? 0 : jPlaylist.current;
  var isWithinPlaylist = jPlaylist.current + 1 < jPlaylist.playlist.length;
  var playNow = true;

  if (!isWithinPlaylist && jPlaylist.loop === 'off') {
    playNow = false;
  }

  return {
    current: isWithinPlaylist ? jPlaylist.current + 1 : current,
    playNow: playNow
  };
};

var previous = function previous(jPlaylist) {
  var current = jPlaylist.loopOnPrevious ? jPlaylist.playlist.length - 1 : jPlaylist.current;

  if (jPlaylist.current - 1 >= 0) {
    current = jPlaylist.current - 1;
  }

  return {
    current: current,
    playNow: true
  };
};

var updateJPlaylist = function updateJPlaylist(state, action, fn) {
  var value = fn(state[action.id], action);
  var jPlaylist = state[action.id];

  return _extends({}, state, _defineProperty({}, action.id, _extends({}, jPlaylist, value)));
};

var reducer = function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _initializeOptions.initialState;
  var action = arguments[1];

  var updateValue = function updateValue(fn) {
    return updateJPlaylist(state, action, fn);
  };

  switch (action.type) {
    case _constants.actionNames.SET_OPTION:
      return updateValue(setOption);
    case _constants.actionNames.SET_PLAYLIST:
      return updateValue(setPlaylist);
    case _constants.actionNames.ADD:
      return updateValue(add);
    case _constants.actionNames.REMOVE:
      return updateValue(remove);
    case _constants.actionNames.CLEAR:
      return updateValue(clear);
    case _constants.actionNames.SELECT:
      return updateValue(select);
    case _constants.actionNames.PLAY:
      return updateValue(play);
    case _constants.actionNames.SHUFFLE:
      return updateValue(shuffle);
    case _constants.actionNames.NEXT:
      return updateValue(next);
    case _constants.actionNames.PREVIOUS:
      return updateValue(previous);
    default:
      return state;
  }
};

exports.default = reducer;